Scriptname kuBFAIQS extends Quest Conditional
{kuertee battle fatigue and injuries quest script}

Function debugTrace (String debugMessage, Int debugLevel = 1, Bool notify = False)
	If debugMode && debugMode >= debugLevel
		If notify
			Debug.Notification ("!kuBFAIQS." + debugMessage)
		EndIf
		Debug.Trace ("!kuBFAIQS." + debugMessage)
	EndIf
EndFunction

Bool Property started Auto
Float version
Float lastVersion
Float lastOnUpdateTimeUpdate
Bool Property resetNow Auto
Int Property debugMode Auto
Int Property testCase Auto
Bool stoppedThenStarted
Float updateInterval
Bool Property uninstallNow Auto
Int menuMode
Bool inited
Message Property uninstallMessage Auto
MQ00QuestScript Property questMQ00 Auto

Event OnInit ()
	If !inited
		inited = True
		Int oldDebugMode = debugMode
		;show debugs in the log during initialisations. the current debugMode is saved.
		;and the debugMode set high, so that debugs are logged.
		debugMode = 10
		debugTrace ("OnInit")
		RegisterForSingleUpdate (1)
		;restore the previous debugMode.
		debugMode = oldDebugMode
	EndIf
EndEvent

Event OnMenuMode (Int id = 1)
	If menuMode != id
		menuMode = id
		debugTrace ("OnMenuMode " + menuMode)
	EndIf
EndEvent

Event OnUpdate ()
	;set version and update interval
	version = 0.621
	updateInterval = 1
	If questMQ00.GetStage () >= 5
		Float timeNow = Utility.GetCurrentRealTime ()
		If !menuMode && (Math.abs (timeNow - lastOnUpdateTimeUpdate) > 60 || timeNow < 10)
			;if current number of seconds is > 1 minute from the last recorded second in the game, it's
			;likely that the game was reloaded.
			initGameLoaded ()

		ElseIf version != lastVersion || resetNow
			;there is a new version. or the player manually reset the mod by typing in the console:
			;SETPQV <modName> RESETNOW TRUE
			initGameLoaded ()

		ElseIf uninstallNow
			debugTrace ("OnUpdate uninstall")
			uninstallNow = False
			uninstall ()
			debugTrace ("uninstalled", 0, True)
			Stop ()

		Else
			;tick () is the main update function.
			;add your code in that function instead of here, to keep this starting block free of
			;mod-specific code.
			tick ()
			;register only for a single update.
			;if RegisterForUpdate () is used, and the mod is deactivated, the Event manager will keep
			;send OnUpdate () Event to the deactivated mod. problems will occur.
			;with RegisterForSingleUpdate (), if the mod is deactivated, the Event manager will fail
			;only once.
			RegisterForSingleUpdate (updateInterval)
		EndIf
		;save the current number of seconds since game-start.
		lastOnUpdateTimeUpdate = timeNow
		menuMode = 0

	Else
		RegisterForSingleUpdate (updateInterval)
	EndIf
EndEvent

Function initGameLoaded ()
	Int oldDebugMode = debugMode
	;show debugs in the log during initialisations.
	debugMode = 10
	Bool newReset
	; Debug.Notification ("initGameLoaded " + version + " != " + lastVersion + " = " + (version != lastVersion))
	If version != lastVersion || !started || resetNow
		; Debug.Notification ("initGameLoaded " + stoppedThenStarted)
		If lastVersion && !stoppedThenStarted
			;if the mod was previously installed (i.e. lastVersion is set), then reset the mod
			;calling Stop () and Start ().
			;because aliases are only ever filled at OnInit () Event, this is needed in case the
			;updates adds or removes new aliases.
			debugTrace ("initGameLoaded reset from v" + lastVersion + " to v" + version)
			If !resetNow
				savePreviousData ()
			EndIf
			;do uninstall procedure
			uninstall (True)
			inited = False
			UnregisterForUpdate ()
			Stop ()
			Utility.Wait (1)
			;Start () will reinitiate the OnInit () Event.
			Start ()
			stoppedThenStarted = True
			newReset = True

		Else
			;this is a new mod activation or a player initiated reset.
			debugTrace ("initGameLoaded init v" + version, 1)
			stoppedThenStarted = False
			lastVersion = version
			resetNow = False
			initNewVersion ()
			started = True
		EndIf
	EndIf

	If !newReset
		;if the game wasn't reset above (i.e. Stop () and Start () wasn't called), then this
		;must be a normal game load.
		debugTrace ("initGameLoaded loaded v" + version)
		initNewGameLoad ()
		;RegisterForSingleUpdate () needed here because
		RegisterForSingleUpdate (1)
	EndIf
	debugMode = oldDebugMode
EndFunction

Function savePreviousData ()
	debugTrace ("savePreviousData")
	kuBFAIBackUpDataQ.armInjuriesValue = armInjuriesValue
	kuBFAIBackUpDataQ.bodyInjuriesValue = bodyInjuriesValue
	kuBFAIBackUpDataQ.headInjuriesValue = headInjuriesValue
	kuBFAIBackUpDataQ.legInjuriesValue = legInjuriesValue
	kuBFAIBackUpDataQ.battleFatiguesValue = battleFatiguesValue
	kuBFAIBackUpDataQ.userChanceInjuries0Arm = userChanceInjuries0Arm
	kuBFAIBackUpDataQ.userChanceInjuries1Body = userChanceInjuries1Body
	kuBFAIBackUpDataQ.userChanceInjuries2Leg = userChanceInjuries2Leg
	kuBFAIBackUpDataQ.userChanceInjuries3Head = userChanceInjuries3Head
	kuBFAIBackUpDataQ.userSecondsToPoolDamage = userSecondsToPoolDamage
	kuBFAIBackUpDataQ.userInjuriesToHealPerHourSlept = userInjuriesToHealPerHourSlept
	kuBFAIBackUpDataQ.userHealthToReceiveBattleFatigue = userHealthToReceiveBattleFatigue
	kuBFAIBackUpDataQ.userDamagePercToReceiveInjury = userDamagePercToReceiveInjury
	kuBFAIBackUpDataQ.userBattleFatigueIncrementalPerHit = userBattleFatigueIncrementalPerHit
	kuBFAIBackUpDataQ.userIncreasedInjuryEffects = userIncreasedInjuryEffects
	kuBFAIBackUpDataQ.gameTimeLastArmTreatment = gameTimeLastArmTreatment
	kuBFAIBackUpDataQ.gameTimeLastBodyTreatment = gameTimeLastBodyTreatment
	kuBFAIBackUpDataQ.gameTimeLastHeadTreatment = gameTimeLastHeadTreatment
	kuBFAIBackUpDataQ.gameTimeLastLegTreatment = gameTimeLastLegTreatment
	kuBFAIBackUpDataQ.userTreatmentTime = userTreatmentTime
	; kuBFAIBackUpDataQ.userCostPerSupplyTreatment = userCostPerSupplyTreatment
	; kuBFAIBackUpDataQ.userMaxTreatmentsInFirstAidKit = userMaxTreatmentsInFirstAidKit
	; kuBFAIBackUpDataQ.firstAidSupplies = firstAidSupplies
	kuBFAIBackUpDataQ.userInjuriesToBattleFatigue = userInjuriesToBattleFatigue
	kuBFAIBackUpDataQ.lastVersion = lastVersion
	kuBFAIBackUpDataQ.userHealerRemoveInjuriesTime = userHealerRemoveInjuriesTime
	kuBFAIBackUpDataQ.dataSaved = True
EndFunction

Function loadPreviousData ()
	debugTrace ("loadPreviousData")
	armInjuriesValue = kuBFAIBackUpDataQ.armInjuriesValue
	bodyInjuriesValue = kuBFAIBackUpDataQ.bodyInjuriesValue
	headInjuriesValue = kuBFAIBackUpDataQ.headInjuriesValue
	legInjuriesValue = kuBFAIBackUpDataQ.legInjuriesValue
	battleFatiguesValue = kuBFAIBackUpDataQ.battleFatiguesValue
	userChanceInjuries0Arm = kuBFAIBackUpDataQ.userChanceInjuries0Arm
	userChanceInjuries1Body = kuBFAIBackUpDataQ.userChanceInjuries1Body
	userChanceInjuries2Leg = kuBFAIBackUpDataQ.userChanceInjuries2Leg
	userChanceInjuries3Head = kuBFAIBackUpDataQ.userChanceInjuries3Head
	userSecondsToPoolDamage = kuBFAIBackUpDataQ.userSecondsToPoolDamage
	userInjuriesToHealPerHourSlept = kuBFAIBackUpDataQ.userInjuriesToHealPerHourSlept
	userHealthToReceiveBattleFatigue = kuBFAIBackUpDataQ.userHealthToReceiveBattleFatigue
	userDamagePercToReceiveInjury = kuBFAIBackUpDataQ.userDamagePercToReceiveInjury
	userBattleFatigueIncrementalPerHit = kuBFAIBackUpDataQ.userBattleFatigueIncrementalPerHit
	gameTimeLastArmTreatment = kuBFAIBackUpDataQ.gameTimeLastArmTreatment
	gameTimeLastBodyTreatment = kuBFAIBackUpDataQ.gameTimeLastBodyTreatment
	gameTimeLastHeadTreatment = kuBFAIBackUpDataQ.gameTimeLastHeadTreatment
	gameTimeLastLegTreatment = kuBFAIBackUpDataQ.gameTimeLastLegTreatment
	userInjuriesToBattleFatigue = kuBFAIBackUpDataQ.userInjuriesToBattleFatigue
	If kuBFAIBackUpDataQ.lastVersion < 0.615
		userIncreasedInjuryEffects = True
		userTreatmentTime = 0.25
		userInjuriesToBattleFatigue = 5
		; userCostPerSupplyTreatment = 100
		; userMaxTreatmentsInFirstAidKit = 25
		; firstAidSupplies = userMaxTreatmentsInFirstAidKit
	Else
		userIncreasedInjuryEffects = kuBFAIBackUpDataQ.userIncreasedInjuryEffects
		userTreatmentTime = kuBFAIBackUpDataQ.userTreatmentTime
		userInjuriesToBattleFatigue = kuBFAIBackUpDataQ.userInjuriesToBattleFatigue
		; userCostPerSupplyTreatment = kuBFAIBackUpDataQ.userCostPerSupplyTreatment
		; userMaxTreatmentsInFirstAidKit = kuBFAIBackUpDataQ.userMaxTreatmentsInFirstAidKit
		; firstAidSupplies = kuBFAIBackUpDataQ.firstAidSupplies
	EndIf
	If kuBFAIBackUpDataQ.lastVersion < 0.621
		userHealerRemoveInjuriesTime = 24
	Else
		userHealerRemoveInjuriesTime = kuBFAIBackUpDataQ.userHealerRemoveInjuriesTime
	EndIf
	kuBFAIBackUpDataQ.dataSaved = False
EndFunction

Function initNewVersion ()
	If !started
		userIncreasedInjuryEffects = True
		userChanceInjuries0Arm = 25
		userChanceInjuries1Body = 65
		userChanceInjuries2Leg = 90
		userChanceInjuries3Head = 100
		userDamageToReceiveInjury = 25
		userDamagePercToReceiveInjury = 0.25
		userSecondsToPoolDamage = 6
		userInjuriesToHealPerHourSlept = 0.5
		userHealthToReceiveBattleFatigue = 25
		userBattleFatigueIncrementalPerHit = 1
		userTreatmentTime = 0.25
		userInjuriesToBattleFatigue = 5
		; userCostPerSupplyTreatment = 100
		; userMaxTreatmentsInFirstAidKit = 25
		; firstAidSupplies = userMaxTreatmentsInFirstAidKit
		userHealerRemoveInjuriesTime = 24
	EndIf
	; removeAllInjuries (Game.GetPlayer ())
	; removeAllTreatments (Game.GetPlayer ())
	If kuBFAIBackUpDataQ.dataSaved
		loadPreviousData ()
	EndIf
	ensureCorrectSpells (Game.GetPlayer (), False)
	ensureTreatmentSpellsCorrect (Game.GetPlayer ())
EndFunction

Function initNewGameLoad ()
	lastTimeDetectInjury = 0
	playerRef = Game.GetPlayer ()
EndFunction

Function uninstall (Bool resetting = False)
	; Int itemCount = Game.GetPlayer ().GetItemCount (kuBFAIFirstAidKit01)
	; If itemCount
	; 	Game.GetPlayer ().RemoveItem (kuBFAIFirstAidKit01, itemCount)
	; EndIf
	removeAllInjuries (Game.GetPlayer ())
	removeAllTreatments (Game.GetPlayer ())
	removeFromVendors ()
	kuBFAIVendorContainersQ.Stop ()
	If !resetting
		If uninstallMessage
			uninstallMessage.Show ()
		Else
			Debug.Notification ("uninstalled")
		EndIf
	EndIf
EndFunction

FormList Property spellArmInjuries Auto
FormList Property spellBodyInjuries Auto
FormList Property spellHeadInjuries Auto
FormList Property spellLegInjuries Auto
FormList Property spellBattleFatigues Auto
Int Property userChanceInjuries0Arm Auto
Int Property userChanceInjuries1Body Auto
Int Property userChanceInjuries2Leg Auto
Int Property userChanceInjuries3Head Auto
Int Property userDamageToReceiveInjury Auto
Int Property userSecondsToPoolDamage Auto
;0.125 = 1 injury healed per 8 hours slept
;0.5 = 4 injuries healed per 8 hours slept
Float Property userInjuriesToHealPerHourSlept Auto
Bool Property bFappliedThisCombat Auto
Float timeSleepStart
Float armInjuriesValue Conditional
Float bodyInjuriesValue Conditional
Float headInjuriesValue Conditional
Float legInjuriesValue Conditional
Float battleFatiguesValue
Float lastHealth
Float lastTimeDetectInjury
Int Property userHealthToReceiveBattleFatigue Auto
Spell appliedArmInjurySpell
Spell appliedBodyInjurySpell
Spell appliedLegInjurySpell
Spell appliedHeadInjurySpell
Spell appliedBattleFatigueSpell
Float Property userDamagePercToReceiveInjury Auto
Float Property userBattleFatigueIncrementalPerHit Auto
Float battleFatigueIncrementalValue
Float lastBaseHealth
FormList Property kuBFAIMagicHealingEffects Auto
Perk Property kuBFAIBattleFatigueMultiplierPerk Auto
Perk Property kuBFAIArmInjuriesMultiplierPerk Auto
Perk Property kuBFAIBodyInjuriesMultiplierPerk Auto
Perk Property kuBFAIHeadInjuriesMultiplierPerk Auto
Perk Property kuBFAILegInjuriesMultiplierPerk Auto
; Int Property magicReliefFromInjuriesActive Auto
Actor playerRef
kuBFAIBackUpDataQS Property kuBFAIBackUpDataQ Auto
Bool Property userIncreasedInjuryEffects Auto
Float gameTimeNow
Float realTimeNow
Idle idleToPlay
Int Property userInjuriesToBattleFatigue Auto
MiscObject Property Gold001 Auto
ImageSpaceModifier Property kuBFAIWakeFromHealingISMod Auto
ImageSpaceModifier Property kuBFAIReceiveInjuryISMod Auto
MiscObject Property PaperRoll Auto
Ingredient Property BirdEgg01 Auto
Ingredient Property MountainFlower01Blue Auto
Ingredient Property MothWingMonarch Auto
Ingredient Property Wheat Auto
Ingredient Property DaedraHeart Auto
Ingredient Property HawkFeathers Auto

Function tick ()
	gameTimeNow = Utility.GetCurrentGameTime () * 24
	realTimeNow = Utility.GetCurrentRealTime ()

	If !kuBFAIVendorContainersQ.IsRunning ()
		kuBFAIVendorContainersQ.Start ()
	EndIf
	If userIncreasedInjuryEffects && !playerRef.HasPerk (kuBFAIBattleFatigueMultiplierPerk)
		playerRef.AddPerk (kuBFAIBattleFatigueMultiplierPerk)
		playerRef.AddPerk (kuBFAIArmInjuriesMultiplierPerk)
		playerRef.AddPerk (kuBFAIBodyInjuriesMultiplierPerk)
		playerRef.AddPerk (kuBFAIHeadInjuriesMultiplierPerk)
		playerRef.AddPerk (kuBFAILegInjuriesMultiplierPerk)
		ensureCorrectSpells (Game.GetPlayer (), False)
	EndIf
	detectInjury (playerRef)
	; checkTreatmentEffectEnd (playerRef)
	ensureTreatmentSpellsCorrect (playerRef)
	If !playerRef.IsInCombat ()
		bFappliedThisCombat = False
	EndIf
	checkIdleToPlay (playerRef)
	checkVendors ()

	If testCase
		doTest ()
	EndIf
EndFunction

Function checkIdleToPlay (Actor actorRef)
	If idleToPlay
		actorRef.Playidle (idleToPlay)
		idleToPlay = None
	EndIf
EndFunction

Function doTest ()
	Int oldDebugMode = debugMode
	debugMode = 10
	If testCase == 1
		;addInjury
		addInjury (Game.GetPlayer ())

	ElseIf testCase == 2
		;get ingredients to make bandages
		playerRef.AddItem (PaperRoll, 10)
		playerRef.AddItem (BirdEgg01, 10)
		playerRef.AddItem (MountainFlower01Blue, 10)
		playerRef.AddItem (MothWingMonarch, 10)
		playerRef.AddItem (Wheat, 10)
		playerRef.AddItem (DaedraHeart, 10)
		playerRef.AddItem (HawkFeathers, 10)

	ElseIf testCase == 3
		;get player-made bandages
		playerRef.AddItem (kuBFAIBandages25Perc, 5)
	EndIf
	debugTrace ("doTest " + testCase + " done", 10, True)
	testCase = 0
	debugMode = oldDebugMode
EndFunction

Event OnSleepStart (Float sleepStartTime, Float sleepDuration)
	timeSleepStart = Utility.GetCurrentGameTime () * 24
EndEvent

Event OnSleepStop (Bool interrupted)
	gameTimeNow = Utility.GetCurrentGameTime () * 24
	Int hoursSlept = (gameTimeNow - timeSleepStart) as Int
	healInjuries (Game.GetPlayer (), hoursSlept)
	ensureTreatmentSpellsCorrect (Game.GetPlayer ())
EndEvent

Function detectInjury (Actor actorRef)
	debugTrace ("detectInjury " + actorRef, 2)
	Float health = actorRef.GetActorValue ("Health")
	If lastHealth == 0
		lastHealth = health
	EndIf
	Float baseHealth = actorRef.GetBaseActorValue ("Health")
	If lastBaseHealth == 0
		lastBaseHealth = baseHealth
	EndIf

	If health < lastHealth && (Math.Abs (health - lastHealth) - Math.Abs (baseHealth - lastBaseHealth) > 1)
		Float timeToPoolDamage = userSecondsToPoolDamage
		Int level = actorRef.GetLevel ()
		timeToPoolDamage -= (level / 50) * (userSecondsToPoolDamage * 0.5)
		Float healthToInjury = baseHealth * userDamagePercToReceiveInjury
		Float healthLoss = lastHealth - health
		debugTrace ("detectInjury health " + lastHealth + " - " + health + " = " + healthLoss + " > " + healthToInjury)
		Float timeElapsed = realTimeNow - lastTimeDetectInjury
		debugTrace ("detectInjury elapsed time " + realTimeNow + " - " + lastTimeDetectInjury + " = " + timeElapsed + " > " + timeToPoolDamage)
		;If healthLoss > healthToInjury || (timeElapsed > timeToPoolDamage)
		;	lastTimeDetectInjury = realTimeNow
		;	If healthLoss > healthToInjury
		;		addInjury (actorRef)
		;	EndIf
		;	lastHealth = health
		;EndIf
		If healthLoss > healthToInjury && timeElapsed < timeToPoolDamage
			addInjury (actorRef)
			lastTimeDetectInjury = realTimeNow
			lastHealth = health

			If actorRef.IsInCombat () && userInjuriesToBattleFatigue
				battleFatigueIncrementalValue += userBattleFatigueIncrementalPerHit
			EndIf
		EndIf

		If timeElapsed > timeToPoolDamage
			lastTimeDetectInjury = realTimeNow
			lastHealth = health
		EndIf

	Else
		lastHealth = health
		lastBaseHealth = baseHealth
		lastTimeDetectInjury = realTimeNow
	EndIf

	If actorRef.IsInCombat ()
		If !bFappliedThisCombat && health < userHealthToReceiveBattleFatigue
			bFappliedThisCombat = True
			addBattleFatigue (actorRef)

		ElseIf battleFatigueIncrementalValue > userInjuriesToBattleFatigue
			;in increments of 5
			battleFatigueIncrementalValue = 0
			addBattleFatigue (actorRef)
		EndIf
	EndIf
EndFunction

Function addInjury (Actor actorRef)
	Int random = Utility.RandomInt (0, 99)
	Bool receivedInjury
	If userChanceInjuries0Arm && random < userChanceInjuries0Arm
		receivedInjury = True
		armInjuriesValue += 1
		appliedArmInjurySpell = setInjuries (actorRef, armInjuriesValue, spellArmInjuries, True)
		gameTimeLastArmTreatment = 0
		debugTrace ("addInjury armInjuriesValue " + armInjuriesValue)

	ElseIf userChanceInjuries1Body && random < userChanceInjuries1Body
		receivedInjury = True
		bodyInjuriesValue += 1
		appliedBodyInjurySpell = setInjuries (actorRef, bodyInjuriesValue, spellBodyInjuries, True)
		gameTimeLastBodyTreatment = 0
		debugTrace ("addInjury bodyInjuriesValue " + bodyInjuriesValue)

	ElseIf userChanceInjuries2Leg && random < userChanceInjuries2Leg
		receivedInjury = True
		legInjuriesValue += 1
		appliedLegInjurySpell = setInjuries (actorRef, legInjuriesValue, spellLegInjuries, True)
		gameTimeLastLegTreatment = 0
		debugTrace ("addInjury legInjuriesValue " + legInjuriesValue)

	ElseIf userChanceInjuries3Head && random < userChanceInjuries3Head
		receivedInjury = True
		headInjuriesValue += 1
		appliedHeadInjurySpell = setInjuries (actorRef, headInjuriesValue, spellHeadInjuries, True)
		gameTimeLastHeadTreatment = 0
		debugTrace ("addInjury headInjuriesValue " + headInjuriesValue)
	EndIf

	If receivedInjury
		random = Utility.RandomInt (0, 99)
		If random < 33 && idleReceiveInjury00
			actorRef.PlayIdle (idleReceiveInjury00)
		ElseIf random < 66 && idleReceiveInjury01
			actorRef.PlayIdle (idleReceiveInjury01)
		ElseIf idleReceiveInjury02
			actorRef.PlayIdle (idleReceiveInjury02)
		EndIf
	EndIf
EndFunction

Function addBattleFatigue (Actor actorRef)
	battleFatiguesValue += 1
	appliedBattleFatigueSpell = setInjuries (actorRef, battleFatiguesValue, spellBattleFatigues, True)
	debugTrace ("!addBattleFatigue battleFatiguesValue " + battleFatiguesValue)
EndFunction

Function healInjuries (Actor actorRef, Int hoursSlept)
	If armInjuriesValue < 7
		armInjuriesValue -= (userInjuriesToHealPerHourSlept * hoursSlept) as Int
	EndIf
	If bodyInjuriesValue < 7
		bodyInjuriesValue -= (userInjuriesToHealPerHourSlept * hoursSlept) as Int
	EndIf
	If headInjuriesValue < 7
		headInjuriesValue -= (userInjuriesToHealPerHourSlept * hoursSlept) as Int
	EndIf
	If legInjuriesValue < 7
		legInjuriesValue -= (userInjuriesToHealPerHourSlept * hoursSlept) as Int
	EndIf
	battleFatiguesValue -= (userInjuriesToHealPerHourSlept * hoursSlept) as Int
	battleFatigueIncrementalValue = 0
	If armInjuriesValue < 0
		armInjuriesValue = 0
	EndIf
	If bodyInjuriesValue < 0
		bodyInjuriesValue = 0
	EndIf
	If headInjuriesValue < 0
		headInjuriesValue = 0
	EndIf
	If legInjuriesValue < 0
		legInjuriesValue = 0
	EndIf
	If battleFatiguesValue < 0
		battleFatiguesValue = 0
	EndIf
	ensureCorrectSpells (actorRef, False)
EndFunction

Function healerRemoveInjuries (Actor actorSpeaker, Actor actorRef)
	debugTrace ("healerRemoveInjuries")
	healAllInjuries (actorRef)
	If actorRef == playerRef
		actorRef.RemoveItem (Gold001, 1000, False, actorSpeaker)
		applyHealISMod ()
		GameHour.Mod (userHealerRemoveInjuriesTime)
	EndIf
EndFunction

Function applyHealISMod ()
	kuBFAIWakeFromHealingISMod.ApplyCrossFade (2)
	Utility.Wait (3)
	ImageSpaceModifier.RemoveCrossFade (2)
EndFunction

Function healAllInjuries (Actor actorRef)
	debugTrace ("heallAllInjuries " + actorRef)
	armInjuriesValue = 0
	bodyInjuriesValue = 0
	headInjuriesValue = 0
	legInjuriesValue = 0
	battleFatiguesValue = 0
	gameTimeLastArmTreatment = 0
	gameTimeLastBodyTreatment = 0
	gameTimeLastHeadTreatment = 0
	gameTimeLastLegTreatment = 0
	treatmentArm25Perc = 0
	treatmentBody25Perc = 0
	treatmentHead25Perc = 0
	treatmentLeg25Perc = 0
	ensureCorrectSpells (actorRef, False)
EndFunction

Function ensureCorrectSpells (Actor actorRef, Bool showMessage = False)
	appliedArmInjurySpell = setInjuries (actorRef, armInjuriesValue, spellArmInjuries, showMessage)
	appliedBodyInjurySpell = setInjuries (actorRef, bodyInjuriesValue, spellBodyInjuries, showMessage)
	appliedHeadInjurySpell = setInjuries (actorRef, headInjuriesValue, spellHeadInjuries, showMessage)
	appliedLegInjurySpell = setInjuries (actorRef, legInjuriesValue, spellLegInjuries, showMessage)
	appliedBattleFatigueSpell = setInjuries (actorRef, battleFatiguesValue, spellBattleFatigues, showMessage)
EndFunction

Spell Function setInjuries (Actor actorRef, Float value, FormList spells, Bool showMessage = False)
	;remove all previous spells first
	Spell injuryRef = None
	If spells
		Int i = 0

		While i < spells.GetSize ()
			injuryRef = spells.GetAt (i) as Spell
			actorRef.RemoveSpell (injuryRef)
			i += 1
		EndWhile

		If value > 0
			If value > spells.GetSize ()
				value = spells.GetSize ()
			EndIf
			injuryRef = spells.GetAt ((value - 1) as Int) as Spell
			If injuryRef
				actorRef.AddSpell (injuryRef, showMessage)
				If showMessage
					If actorRef == playerRef
						debugTrace ("setInjuries kuBFAIReceiveInjuryISMod.ApplyCrossFade (1)")
						kuBFAIReceiveInjuryISMod.ApplyCrossFade (1)
						Utility.Wait (1)
						ImageSpaceModifier.RemoveCrossFade (1)
					EndIf
				EndIf
			EndIf
		EndIf
	EndIf
	Return injuryRef
EndFunction

Function removeAllInjuries (Actor actorRef)
	;remove all previous spells first
	Spell injuryRef
	Int i = 0
	While i < spellArmInjuries.GetSize ()
		injuryRef = spellArmInjuries.GetAt (i) as Spell
		actorRef.RemoveSpell (injuryRef)
		i += 1
	EndWhile
	i = 0
	While i < spellBodyInjuries.GetSize ()
		injuryRef = spellBodyInjuries.GetAt (i) as Spell
		actorRef.RemoveSpell (injuryRef)
		i += 1
	EndWhile
	i = 0
	While i < spellHeadInjuries.GetSize ()
		injuryRef = spellHeadInjuries.GetAt (i) as Spell
		actorRef.RemoveSpell (injuryRef)
		i += 1
	EndWhile
	i = 0
	While i < spellLegInjuries.GetSize ()
		injuryRef = spellLegInjuries.GetAt (i) as Spell
		actorRef.RemoveSpell (injuryRef)
		i += 1
	EndWhile
	i = 0
	While i < spellBattleFatigues.GetSize ()
		injuryRef = spellBattleFatigues.GetAt (i) as Spell
		actorRef.RemoveSpell (injuryRef)
		i += 1
	EndWhile
EndFunction

; Function OnMagicEffectApply (Actor actorRef, ObjectReference akCaster, MagicEffect akEffect)
; 	If !magicReliefFromInjuriesActive
; 		If kuBFAIMagicHealingEffects.HasForm (akEffect)
; 			;actorRef applied magically healing
; 		EndIf
; 	EndIf
; EndFunction

; Float Property firstAidSupplies Auto
Message Property kuBFAINoInjuriesToTreat Auto
Message Property kuBFAINoTreatmentsLeft Auto
Message Property kuBFAITreatInjuries Auto
Spell Property kuBFAITreatedArmInjuries Auto
Spell Property kuBFAITreatedBodyInjuries Auto
Spell Property kuBFAITreatedHeadInjuries Auto
Spell Property kuBFAITreatedLegInjuries Auto
Float Property gameTimeLastArmTreatment Auto
Float Property gameTimeLastBodyTreatment Auto
Float Property gameTimeLastHeadTreatment Auto
Float Property gameTimeLastLegTreatment Auto
Float Property userTreatmentTime Auto;in hours
; Int Property userCostPerSupplyTreatment Auto;for each treatment
; Int Property userMaxTreatmentsInFirstAidKit Auto
Quest Property kuBFAIVendorContainersQ Auto
LeveledItem Property kuBFAILeveledFirstAidKits Auto
ReferenceAlias [] Property vendorChestArray Auto
Message Property kuBFAITreatmentInCombat Auto
GlobalVariable Property GameHour Auto
Float gameTimeVendorsReset
MiscObject Property kuBFAIBandages Auto
Idle Property idleTreatArmInjures Auto
Idle Property idleTreatBodyInjures Auto
Idle Property idleTreatHeadInjures Auto
Idle Property idleTreatLegInjures Auto
Idle Property idleReceiveInjury00 Auto
Idle Property idleReceiveInjury01 Auto
Idle Property idleReceiveInjury02 Auto
Spell Property kuBFAIHalfTreatedArmInjuries Auto
Spell Property kuBFAIHalfTreatedBodyInjuries Auto
Spell Property kuBFAIHalfTreatedHeadInjuries Auto
Spell Property kuBFAIHalfTreatedLegInjuries Auto
MiscObject Property kuBFAIBandages25Perc Auto
Int treatmentArm25Perc;0=normal from apothecary, 1=25% from crafted bandages
Int treatmentBody25Perc
Int treatmentHead25Perc
Int treatmentLeg25Perc
Float Property userHealerRemoveInjuriesTime Auto;in hours

Function treatInjuries (Actor actorRef, MiscObject firstAidKit)
	debugTrace ("treatInjuries " + actorRef)
	gameTimeNow = Utility.GetCurrentGameTime () * 24
	; If firstAidSupplies < 1
	; 	kuBFAINoTreatmentsLeft.Show ()

	If actorRef.IsInCombat ()
		kuBFAITreatmentInCombat.Show ()

	ElseIf !armInjuriesValue && !bodyInjuriesValue && !headInjuriesValue && !legInjuriesValue && !debugMode
		; kuBFAINoInjuriesToTreat.Show (firstAidSupplies)
		kuBFAINoInjuriesToTreat.Show ()

	Else
		Int hours = Math.Floor (userTreatmentTime)
		Float timeLeft = userTreatmentTime - hours
		Int minutes = Math.Floor (timeLeft * 60)
		; Int buttonPressed = kuBFAITreatInjuries.Show (firstAidSupplies, hours, minutes)
		Int buttonPressed = kuBFAITreatInjuries.Show (hours, minutes)
		If buttonPressed < 4
			; firstAidSupplies -= 1
			actorRef.RemoveItem (firstAidKit, 1)
			applyHealISMod ()
			GameHour.Mod (userTreatmentTime)

			If buttonPressed == 0
				gameTimeLastArmTreatment = gameTimeNow
				idleToPlay = idleTreatArmInjures
				If treatmentArm25Perc || playerRef.HasSpell (kuBFAIHalfTreatedArmInjuries) || playerRef.HasSpell (kuBFAITreatedArmInjuries)
					;already half-treated, so apply full treatment from this half
					treatmentArm25Perc = 0
				ElseIf firstAidKit == kuBFAIBandages25Perc
					treatmentArm25Perc = 1
				EndIf

			ElseIf buttonPressed == 1
				gameTimeLastBodyTreatment = gameTimeNow
				idleToPlay = idleTreatBodyInjures
				If treatmentBody25Perc || playerRef.HasSpell (kuBFAIHalfTreatedBodyInjuries) || playerRef.HasSpell (kuBFAITreatedBodyInjuries)
					treatmentBody25Perc = 0
				ElseIf firstAidKit == kuBFAIBandages25Perc
					treatmentBody25Perc = 1
				EndIf

			ElseIf buttonPressed == 2
				gameTimeLastHeadTreatment = gameTimeNow
				idleToPlay = idleTreatHeadInjures
				If treatmentHead25Perc || playerRef.HasSpell (kuBFAIHalfTreatedHeadInjuries) || playerRef.HasSpell (kuBFAITreatedHeadInjuries)
					treatmentHead25Perc = 0
				ElseIf firstAidKit == kuBFAIBandages25Perc
					treatmentHead25Perc = 1
				EndIf

			ElseIf buttonPressed == 3
				gameTimeLastLegTreatment = gameTimeNow
				idleToPlay = idleTreatLegInjures
				If treatmentLeg25Perc || playerRef.HasSpell (kuBFAIHalfTreatedLegInjuries) || playerRef.HasSpell (kuBFAITreatedLegInjuries)
					treatmentLeg25Perc = 0
				ElseIf firstAidKit == kuBFAIBandages25Perc
					treatmentLeg25Perc = 1
				EndIf
			EndIf
		EndIf
	EndIf
EndFunction

; Function checkTreatmentEffectEnd (Actor actorRef)
; 	gameTimeNow = Utility.GetCurrentGameTime ()
; 	If playerRef.hasSpell (kuBFAITreatedArmInjuries) && gameTimeNow - gameTimeLastArmTreatment > 12
; 		gameTimeLastArmTreatment = 0
; 	EndIf
; 	If playerRef.hasSpell (kuBFAITreatedBodyInjuries) && gameTimeNow - gameTimeLastBodyTreatment > 12
; 		gameTimeLastBodyTreatment = 0
; 	EndIf
; 	If playerRef.hasSpell (kuBFAITreatedHeadInjuries) && gameTimeNow - gameTimeLastHeadTreatment > 12
; 		gameTimeLastHeadTreatment = 0
; 	EndIf
; 	If playerRef.hasSpell (kuBFAITreatedLegInjuries) && gameTimeNow - gameTimeLastLegTreatment > 12
; 		gameTimeLastLegTreatment = 0
; 	EndIf
; EndFunction

Function ensureTreatmentSpellsCorrect (Actor actorRef, Bool showMessage = False)
	gameTimeNow = Utility.GetCurrentGameTime () * 24
	debugTrace ("ensureTreatmentSpellsCorrect " + kuBFAIHalfTreatedArmInjuries, 2)
	debugTrace ("ensureTreatmentSpellsCorrect " + kuBFAITreatedArmInjuries, 2)
	debugTrace ("ensureTreatmentSpellsCorrect " + kuBFAIHalfTreatedBodyInjuries, 2)
	debugTrace ("ensureTreatmentSpellsCorrect " + kuBFAITreatedBodyInjuries, 2)
	debugTrace ("ensureTreatmentSpellsCorrect " + kuBFAIHalfTreatedHeadInjuries, 2)
	debugTrace ("ensureTreatmentSpellsCorrect " + kuBFAITreatedHeadInjuries, 2)
	debugTrace ("ensureTreatmentSpellsCorrect " + kuBFAIHalfTreatedLegInjuries, 2)
	debugTrace ("ensureTreatmentSpellsCorrect " + kuBFAITreatedLegInjuries, 2)
	If gameTimeNow - gameTimeLastArmTreatment < 12
		If treatmentArm25Perc && !actorRef.HasSpell (kuBFAIHalfTreatedArmInjuries)
			actorRef.AddSpell (kuBFAIHalfTreatedArmInjuries, showMessage)
		ElseIf !treatmentArm25Perc && !actorRef.HasSpell (kuBFAITreatedArmInjuries)
			actorRef.AddSpell (kuBFAITreatedArmInjuries, showMessage)
			actorRef.RemoveSpell (kuBFAIHalfTreatedArmInjuries)
		EndIf

	Else
		gameTimeLastArmTreatment = 0
		treatmentArm25Perc = 0
		If actorRef.HasSpell (kuBFAIHalfTreatedArmInjuries)
			actorRef.RemoveSpell (kuBFAIHalfTreatedArmInjuries)
		EndIf
		If actorRef.HasSpell (kuBFAITreatedArmInjuries)
			actorRef.RemoveSpell (kuBFAITreatedArmInjuries)
		EndIf
	EndIf
	If gameTimeNow - gameTimeLastBodyTreatment < 12
		If treatmentBody25Perc && !actorRef.HasSpell (kuBFAIHalfTreatedBodyInjuries)
			actorRef.AddSpell (kuBFAIHalfTreatedBodyInjuries, showMessage)
		ElseIf !treatmentBody25Perc && !actorRef.HasSpell (kuBFAITreatedBodyInjuries)
			actorRef.AddSpell (kuBFAITreatedBodyInjuries, showMessage)
			actorRef.RemoveSpell (kuBFAIHalfTreatedBodyInjuries)
		EndIf

	Else
		gameTimeLastBodyTreatment = 0
		treatmentBody25Perc = 0
		If actorRef.HasSpell (kuBFAIHalfTreatedBodyInjuries)
			actorRef.RemoveSpell (kuBFAIHalfTreatedBodyInjuries)
		EndIf
		If actorRef.HasSpell (kuBFAITreatedBodyInjuries)
			actorRef.RemoveSpell (kuBFAITreatedBodyInjuries)
		EndIf
	EndIf
	If gameTimeNow - gameTimeLastHeadTreatment < 12
		If treatmentHead25Perc && !actorRef.HasSpell (kuBFAIHalfTreatedHeadInjuries)
			actorRef.AddSpell (kuBFAIHalfTreatedHeadInjuries, showMessage)
		ElseIf !treatmentHead25Perc && !actorRef.HasSpell (kuBFAITreatedHeadInjuries)
			actorRef.AddSpell (kuBFAITreatedHeadInjuries, showMessage)
			actorRef.RemoveSpell (kuBFAIHalfTreatedHeadInjuries)
		EndIf

	Else
		gameTimeLastHeadTreatment = 0
		treatmentHead25Perc = 0
		If actorRef.HasSpell (kuBFAIHalfTreatedHeadInjuries)
			actorRef.RemoveSpell (kuBFAIHalfTreatedHeadInjuries)
		EndIf
		If actorRef.HasSpell (kuBFAITreatedHeadInjuries)
			actorRef.RemoveSpell (kuBFAITreatedHeadInjuries)
		EndIf
	EndIf
	If gameTimeNow - gameTimeLastLegTreatment < 12
		If treatmentLeg25Perc && !actorRef.HasSpell (kuBFAIHalfTreatedLegInjuries)
			actorRef.AddSpell (kuBFAIHalfTreatedLegInjuries, showMessage)
		ElseIf !treatmentLeg25Perc && !actorRef.HasSpell (kuBFAITreatedLegInjuries)
			actorRef.AddSpell (kuBFAITreatedLegInjuries, showMessage)
			actorRef.RemoveSpell (kuBFAIHalfTreatedLegInjuries)
		EndIf

	Else
		gameTimeLastLegTreatment = 0
		treatmentLeg25Perc = 0
		If actorRef.HasSpell (kuBFAIHalfTreatedLegInjuries)
			actorRef.RemoveSpell (kuBFAIHalfTreatedLegInjuries)
		EndIf
		If actorRef.HasSpell (kuBFAITreatedLegInjuries)
			actorRef.RemoveSpell (kuBFAITreatedLegInjuries)
		EndIf
	EndIf
EndFunction

Function removeAllTreatments (Actor actorRef)
	actorRef.RemoveSpell (kuBFAITreatedArmInjuries)
	actorRef.RemoveSpell (kuBFAITreatedBodyInjuries)
	actorRef.RemoveSpell (kuBFAITreatedHeadInjuries)
	actorRef.RemoveSpell (kuBFAITreatedLegInjuries)
	actorRef.RemoveSpell (kuBFAIHalfTreatedArmInjuries)
	actorRef.RemoveSpell (kuBFAIHalfTreatedBodyInjuries)
	actorRef.RemoveSpell (kuBFAIHalfTreatedHeadInjuries)
	actorRef.RemoveSpell (kuBFAIHalfTreatedLegInjuries)
EndFunction

Function checkVendors ()
	gameTimeNow = Utility.GetCurrentGameTime () * 24
	debugTrace ("checkVendors " + gameTimeNow + " - " + gameTimeVendorsReset)
	If !kuBFAIVendorContainersQ.IsRunning ()
		kuBFAIVendorContainersQ.Start ()
		gameTimeVendorsReset = gameTimeNow

	ElseIf gameTimeNow - gameTimeVendorsReset > 24
		removeFromVendors ()
		kuBFAIVendorContainersQ.Stop ()
		gameTimeVendorsReset = gameTimeNow
	EndIf
EndFunction

Function removeFromVendors ()
	If kuBFAIVendorContainersQ.IsRunning ()
		Int i = 0
		ReferenceAlias vendorChestAlias
		ObjectReference vendorChestRef
		Int itemCount
		While i < vendorChestArray.Length
			vendorChestAlias = vendorChestArray [i] as ReferenceAlias
			If vendorChestAlias
				vendorChestRef = vendorChestAlias.GetReference ()
				If vendorChestRef
					itemCount = vendorChestRef.GetItemCount (kuBFAIBandages)
					If itemCount
						vendorChestRef.RemoveItem (kuBFAIBandages, itemCount)
						; debugTrace ("removeFromVendors removing first aid kits from " + vendorChestRef + " " + itemCount)

					EndIf
					; itemCount = vendorChestRef.GetItemCount (kuBFAIFirstAidKitPotion)
					; If itemCount
					; 	vendorChestRef.RemoveItem (kuBFAIFirstAidKitPotion, itemCount)
					; 	debugTrace ("removeFromVendors removing first aid kits potion from " + vendorChestRef + " " + itemCount)
					; EndIf
				EndIf
			EndIf
			i += 1
		EndWhile
	EndIf
EndFunction

Message Property kuBFAIMakeBandages Auto
Function makeBandages ()
	Int button = kuBFAIMakeBandages.Show ()
	Ingredient ingredientToRemove
	Int count
	Int bandages = 1
	If button == 0
		ingredientToRemove = BirdEgg01
		count = 1
	ElseIf button == 1
		ingredientToRemove = MountainFlower01Blue
		count = 3
	ElseIf button == 2
		ingredientToRemove = MothWingMonarch
		count = 3
	ElseIf button == 3
		ingredientToRemove = Wheat
		count = 3
	ElseIf button == 4
		ingredientToRemove = DaedraHeart
		count = 1
		bandages = 2
	ElseIf button == 5
		ingredientToRemove = HawkFeathers
		count = 3
	EndIf
	If count
		playerRef.RemoveItem (PaperRoll, 1, True)
		playerRef.RemoveItem (ingredientToRemove, count, True)
		playerRef.AddItem (kuBFAIBandages25Perc, bandages)
	EndIf
EndFunction

Message Property kuBFAIMakeBandages2 Auto
Message Property kuBFAINoPotions Auto
Potion Property RestoreHealth01 Auto
Potion Property RestoreHealth02 Auto
Potion Property RestoreHealth03 Auto
Potion Property RestoreHealth04 Auto
Potion Property RestoreHealth05 Auto
Potion Property RestoreHealth06 Auto
Function makeBandages2 ()
	Int potionCount = playerRef.GetItemCount (RestoreHealth01)
	potionCount += playerRef.GetItemCount (RestoreHealth02)
	potionCount += playerRef.GetItemCount (RestoreHealth03)
	potionCount += playerRef.GetItemCount (RestoreHealth04)
	potionCount += playerRef.GetItemCount (RestoreHealth05)
	potionCount += playerRef.GetItemCount (RestoreHealth06)
	If potionCount < 1
		kuBFAINoPotions.Show ()
	Else
		Int button = kuBFAIMakeBandages2.Show ()
		Potion toRemove
		Int count = 1
		Int bandages
		If button == 0
			toRemove = RestoreHealth01
			bandages = 1
		ElseIf button == 1
			toRemove = RestoreHealth02
			bandages = 2
		ElseIf button == 2
			toRemove = RestoreHealth03
			bandages = 3
		ElseIf button == 3
			toRemove = RestoreHealth04
			bandages = 4
		ElseIf button == 4
			toRemove = RestoreHealth05
			bandages = 5
		ElseIf button == 5
			toRemove = RestoreHealth06
			bandages = 6
		EndIf
		If bandages
			playerRef.RemoveItem (PaperRoll, 1, True)
			playerRef.RemoveItem (toRemove, count, True)
			playerRef.AddItem (kuBFAIBandages25Perc, bandages)
		EndIf
	EndIf
EndFunction

; Actor [] npcs
; Float [] npcHealths
; Int [] npcArmInjuries
; Int [] npcBodyInjuries
; Int [] npcHeadInjuries
; Int [] npcLegInjuries
; Int npcIndex

; Function npcInit (Actor actorRef)
; 	Int i = npcFindInArray (actorRef)
; 	If i == -1 && lastNNPCFindInArrayEmpty > -1 && lastNNPCFindInArrayEmpty < npcs.Length
; 		npcAddToArray (actorRef, lastNNPCFindInArrayEmpty)
; 	EndIf
; EndFunction

; Function npcAddToArray (Actor actorRef, Int index)
; 	npcs [index] = actorRef
; 	npcHealths [index] = actorRef.GetActorValue ("Health")
; 	npcArmInjuries [index] = 0
; 	npcBodyInjuries [index] = 0
; 	npcHeadInjuries [index] = 0
; 	npcLegInjuries [index] = 0
; EndFunction

; Function npcTick ()
; 	Int i = 0
; 	Int npcIndex
; 	Actor npcRef
; 	Float health
; 	While i < 10 && npcIndex < npcs.Length
; 		npcIndex = i + i
; 		If npcIndex < npcs.Length
; 			npcRef = npcs [npcIndex]
; 			If npcRef
; 			EndIf
; 		EndIf
; 		i += 1
; 	EndWhile
; 	If npcIndex > npcs.Length
; 		npcIndex = 0
; 	EndIf
; EndFunction

; Function npcDetectInjuries (Actor actorRef)
; EndFunction

; Function npcClean (Actor actorRef)
; EndFunction

; Int lastNNPCFindInArrayEmpty
; Function npcFindInArray (Actor actorRef, Bool returnEmpty = False)
; 	Int found = -1
; 	Int i = 0
; 	Actor npcRef
; 	Bool exit
; 	Int empty = -1
; 	While i < npcs.Length && !exit
; 		npcRef = npcs [i] as Actor
; 		If npcRef == actorRef
; 			found = i
; 			exit = True
; 		ElseIf !npcRef && empty < 0
; 			empty = i
; 			lastNNPCFindInArrayEmpty = i
; 		EndIf
; 		i += 1
; 	EndWhile
; 	If returnEmpty
; 		found = empty
; 	EndIf
; 	Return found
; EndFunction
